@@ -73,5 +73,9 @@ struct block *block_parse(xmlDocPtr, xmlNsPtr, xmlNodePtr);
 void block_save(struct block *, int, xmlNodePtr);
 /* Transposes a block or a part of it */
 void block_transpose(struct block *, int, int, int, int, int, int);
+/* Expands/shrinks a block or a part of it */
+void block_expandshrink(struct block *, int, int, int, int, int);
+/* Changes or swaps an instrument to another */
+void block_changeinstrument(struct block *, int, int, int, int, int, int, int);
 
 #endif
