@@ -26,6 +26,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <pthread.h>
+#include <gconf/gconf-client.h>
 #include "song.h"
 #include "block.h"
 #include "gui.h"
@@ -52,6 +53,9 @@ struct editor {
   GThread *thread;
   /* Player for this editor */
   struct player *player;
+  /* GConf client for preferences */
+  GConfClient *gconfclient;
+  guint gconfnotifyid;
 };
 
 /* Initializes the editor */
