@@ -24,6 +24,12 @@
 
 #include <libxml/tree.h>
 
+/* MIDI data structure */
+struct midi {
+  /* MIDI device file descriptor */
+  int fd;
+};
+
 struct sysex {
   /* Name */
   char *name;
@@ -36,31 +42,31 @@ struct sysex {
 };
 
 /* Opens a MIDI device */
-int midi_open(char *);
+struct midi *midi_open(char *);
 /* Closes a MIDI device */
-void midi_close(int);
+void midi_close(struct midi *);
 /* Stops a note playing on a MIDI channel using requested velocity */
-void midi_note_off(int, unsigned char, unsigned char, unsigned char);
+void midi_note_off(struct midi *, unsigned char, unsigned char, unsigned char);
 /* Plays a note on a MIDI channel using requested velocity */
-void midi_note_on(int, unsigned char, unsigned char, unsigned char);
+void midi_note_on(struct midi *, unsigned char, unsigned char, unsigned char);
 /* Sets the aftertouch pressure of a note playing on a MIDI channel */
-void midi_aftertouch(int, unsigned char, unsigned char, unsigned char);
+void midi_aftertouch(struct midi *, unsigned char, unsigned char, unsigned char);
 /* Sets the MIDI controller value of a MIDI channel */
-void midi_controller(int, unsigned char, unsigned char, unsigned char);
+void midi_controller(struct midi *, unsigned char, unsigned char, unsigned char);
 /* Send a program change on a MIDI channel */
-void midi_program_change(int, unsigned char, unsigned char);
+void midi_program_change(struct midi *, unsigned char, unsigned char);
 /* Sets the channel pressure of a MIDI channel */
-void midi_channel_pressure(int, unsigned char, unsigned char);
+void midi_channel_pressure(struct midi *, unsigned char, unsigned char);
 /* Sets the pitch wheel value of a MIDI channel */
-void midi_pitch_wheel(int, unsigned char, unsigned short);
+void midi_pitch_wheel(struct midi *, unsigned char, unsigned short);
 /* Sends a MIDI SysEx message */
-void midi_system_exclusive(int, unsigned char *, unsigned short);
+void midi_system_exclusive(struct midi *, unsigned char *, unsigned short);
 /* Send a clock message */
-void midi_clock(int);
+void midi_clock(struct midi *);
 /* Receives a MIDI message */
-unsigned char *midi_read(int);
+unsigned char *midi_read(struct midi *);
 /* Receives a MIDI SysEx message */
-void midi_read_system_exclusive(int, struct sysex *, int);
+void midi_read_system_exclusive(struct midi *, struct sysex *, int);
 /* Sets the length of a MIDI SysEx message */
 void sysex_set_length(struct sysex *, unsigned int);
 /* Sets the auto send flag of a MIDI SysEx message */
