@@ -145,8 +145,8 @@ void block_set_tracks(struct block *block, unsigned int tracks) {
       notes[(l*tracks+t)*2]=block->notes[(l*nt+t)*2];
       notes[(l*tracks+t)*2+1]=block->notes[(l*nt+t)*2+1];
       for(ep=0; ep<block->commandpages; ep++) {
-	commands[ep*2*tracks*nl+(l*tracks+t)*2]=block->commands[ep*2*existingtracks*nl+(l*nt+t)*2];
-	commands[ep*2*tracks*nl+(l*tracks+t)*2+1]=block->commands[ep*2*existingtracks*nl+(l*nt+t)*2+1];
+	commands[ep*2*tracks*nl+(l*tracks+t)*2]=block->commands[ep*2*nt*nl+(l*nt+t)*2];
+	commands[ep*2*tracks*nl+(l*tracks+t)*2+1]=block->commands[ep*2*nt*nl+(l*nt+t)*2+1];
       }
     }
   
@@ -188,8 +188,8 @@ void block_set_length(struct block *block, unsigned int length) {
       notes[(l*nt+t)*2]=block->notes[(l*nt+t)*2];
       notes[(l*nt+t)*2+1]=block->notes[(l*nt+t)*2+1];
       for(ep=0; ep<block->commandpages; ep++) {
-	commands[ep*2*nt*length+(l*nt+t)*2]=block->commands[ep*2*nt*existinglength+(l*nt+t)*2];
-	commands[ep*2*nt*length+(l*nt+t)*2+1]=block->commands[ep*2*nt*existinglength+(l*nt+t)*2+1];
+	commands[ep*2*nt*length+(l*nt+t)*2]=block->commands[ep*2*nt*nl+(l*nt+t)*2];
+	commands[ep*2*nt*length+(l*nt+t)*2+1]=block->commands[ep*2*nt*nl+(l*nt+t)*2+1];
       }
     }
   
@@ -398,7 +398,7 @@ struct block *block_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
       }
       cur=cur->next;
     }
-  } else
+  } else if(cur->type!=XML_COMMENT_NODE)
     fprintf(stderr, "XML error: expected block, got %s\n", cur->name);
 
   return block;
@@ -520,3 +520,110 @@ void block_transpose(struct block *block,
 	}
       }
 }
+
+/* Expands/shrinks a block or a part of it */
+void block_expandshrink(struct block *block, int factor,
+			int starttrack, int startline,
+			int endtrack, int endline) {
+  int l, t;
+
+  if(block==NULL) {
+    fprintf(stderr, "block_expandshrink() called with null block\n");
+    return;
+  }
+
+  if(factor>-2 && factor<2)
+    return;
+
+  /* Bounds checking */
+  if(starttrack<0)
+    starttrack=0;
+  if(startline<0)
+    startline=0;
+  if(starttrack>=block->tracks)
+    starttrack=block->tracks-1;
+  if(startline>=block->length)
+    startline=block->length-1;
+  if(endtrack<starttrack)
+    endtrack=starttrack;
+  if(endline<startline)
+    endline=startline;
+  if(endtrack>=block->tracks)
+    endtrack=block->tracks-1;
+  if(endline>=block->length)
+    endline=block->length-1;
+  
+  if(factor<0) {
+    /* Shrink */
+    for(l=startline; l<=endline; l++)
+      if((l-startline)<(endline+1-startline)/-factor) {
+	for(t=starttrack; t<=endtrack; t++) {
+	  block->notes[(l*block->tracks+t)*2]=block->notes[((startline+(l-startline)*-factor)*block->tracks+t)*2];
+	  block->notes[(l*block->tracks+t)*2+1]=block->notes[((startline+(l-startline)*-factor)*block->tracks+t)*2+1];
+	}
+      } else {
+	for(t=starttrack; t<=endtrack; t++) {
+	  block->notes[(l*block->tracks+t)*2]=0;
+	  block->notes[(l*block->tracks+t)*2+1]=0;
+	}
+      }
+  } else {
+    /* Expand */
+    for(l=endline; l>=startline; l--)
+      if((l-startline)%factor==0) {
+	for(t=starttrack; t<=endtrack; t++) {
+	  block->notes[(l*block->tracks+t)*2]=block->notes[((startline+(l-startline)/factor)*block->tracks+t)*2];
+	  block->notes[(l*block->tracks+t)*2+1]=block->notes[((startline+(l-startline)/factor)*block->tracks+t)*2+1];
+	}
+      } else {
+	for(t=starttrack; t<=endtrack; t++) {
+	  block->notes[(l*block->tracks+t)*2]=0;
+	  block->notes[(l*block->tracks+t)*2+1]=0;
+	}
+      }
+  }
+}
+
+/* Changes an instrument to another */
+void block_changeinstrument(struct block *block, int from, int to, int swap,
+			    int starttrack, int startline,
+			    int endtrack, int endline) {
+  int l, t;
+
+  if(block==NULL) {
+    fprintf(stderr, "block_changeinstrument() called with null block\n");
+    return;
+  }
+
+  /* Bounds checking */
+  if(starttrack<0)
+    starttrack=0;
+  if(startline<0)
+    startline=0;
+  if(starttrack>=block->tracks)
+    starttrack=block->tracks-1;
+  if(startline>=block->length)
+    startline=block->length-1;
+  if(endtrack<starttrack)
+    endtrack=starttrack;
+  if(endline<startline)
+    endline=startline;
+  if(endtrack>=block->tracks)
+    endtrack=block->tracks-1;
+  if(endline>=block->length)
+    endline=block->length-1;
+  
+  if(swap)
+    for(l=endline; l>=startline; l--)
+      for(t=starttrack; t<=endtrack; t++) {
+	if(block->notes[(l*block->tracks+t)*2+1]==from)
+	  block->notes[(l*block->tracks+t)*2+1]=to;
+	else if(block->notes[(l*block->tracks+t)*2+1]==to)
+	  block->notes[(l*block->tracks+t)*2+1]=from;
+      }
+  else
+    for(l=endline; l>=startline; l--)
+      for(t=starttrack; t<=endtrack; t++)
+	if(block->notes[(l*block->tracks+t)*2+1]==from)
+	  block->notes[(l*block->tracks+t)*2+1]=to;
+}
