@@ -27,34 +27,41 @@
 #include "midi.h"
 
 /* Opens a MIDI device */
-int midi_open(char *filename) {
-  int midi;
+struct midi *midi_open(char *filename) {
+  struct midi *midi=(struct midi *)calloc(1, sizeof(struct midi));
 
   /* Open MIDI device:read/write, do not block (for input), auto flush */
-  midi=open(filename, O_RDWR|O_NONBLOCK|O_SYNC);
-  if(midi==-1)
+  midi->fd=open(filename, O_RDWR|O_NONBLOCK);
+  if(midi->fd==-1) {
     fprintf(stderr, "Could not open MIDI device %s.\n", filename);
+    free(midi);
+    midi=NULL;
+  }
 
   return midi;
 }
 
 /* Closes a MIDI device */
-void midi_close(int midi) {
-  if(midi==-1) {
-    fprintf(stderr, "midi_close() called with invalid fd\n");
+void midi_close(struct midi *midi) {
+  if(midi==NULL) {
+    fprintf(stderr, "midi_close() called with null midi\n");
     return;
   }
 
-  close(midi);
+  if(midi->fd!=-1) {
+    fsync(midi->fd);
+    close(midi->fd);
+  }
+  free(midi);
 }
 
 /* Stops a note playing on a MIDI channel using requested velocity */
-void midi_note_off(int midi, unsigned char channel, unsigned char note,
+void midi_note_off(struct midi *midi, unsigned char channel, unsigned char note,
 		   unsigned char velocity) {
   unsigned char data[3];
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_note_off() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_note_off() called with null midi\n");
     return;
   }
 
@@ -62,16 +69,17 @@ void midi_note_off(int midi, unsigned char channel, unsigned char note,
   data[1]=note&0x7f;
   data[2]=velocity&0x7f;
   
-  write(midi, data, 3);
+  write(midi->fd, data, 3);
+  fsync(midi->fd);
 }
 
 /* Plays a note on a MIDI channel using requested velocity */
-void midi_note_on(int midi, unsigned char channel, unsigned char note,
+void midi_note_on(struct midi *midi, unsigned char channel, unsigned char note,
 		   unsigned char velocity) {
   unsigned char data[3];
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_note_on() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_note_on() called with null midi\n");
     return;
   }
 
@@ -79,16 +87,17 @@ void midi_note_on(int midi, unsigned char channel, unsigned char note,
   data[1]=note&0x7f;
   data[2]=velocity&0x7f;
   
-  write(midi, data, 3);
+  write(midi->fd, data, 3);
+  fsync(midi->fd);
 }
 
 /* Sets the aftertouch pressure of a note playing on a MIDI channel */
-void midi_aftertouch(int midi, unsigned char channel, unsigned char note,
+void midi_aftertouch(struct midi *midi, unsigned char channel, unsigned char note,
 		     unsigned char pressure) {
   unsigned char data[3];
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_aftertouch() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_aftertouch() called with null midi\n");
     return;
   }
 
@@ -96,16 +105,17 @@ void midi_aftertouch(int midi, unsigned char channel, unsigned char note,
   data[1]=note&0x7f;
   data[2]=pressure&0x7f;
   
-  write(midi, data, 3);
+  write(midi->fd, data, 3);
+  fsync(midi->fd);
 }
 
 /* Sets the MIDI controller value of a MIDI channel */
-void midi_controller(int midi, unsigned char channel, 
+void midi_controller(struct midi *midi, unsigned char channel, 
 		     unsigned char controller, unsigned char value) {
   unsigned char data[3];
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_controller() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_controller() called with null midi\n");
     return;
   }
 
@@ -113,48 +123,51 @@ void midi_controller(int midi, unsigned char channel,
   data[1]=controller&0x7f;
   data[2]=value&0x7f;
   
-  write(midi, data, 3);
+  write(midi->fd, data, 3);
+  fsync(midi->fd);
 }
 
 /* Send a program change on a MIDI channel */
-void midi_program_change(int midi, unsigned char channel,
+void midi_program_change(struct midi *midi, unsigned char channel,
 			 unsigned char program) {
   unsigned char data[2];
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_program_change() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_program_change() called with null midi\n");
     return;
   }
 
   data[0]=0xc0|channel;
   data[1]=program&0x7f;
   
-  write(midi, data, 2);
+  write(midi->fd, data, 2);
+  fsync(midi->fd);
 }
 
 /* Sets the channel pressure of a MIDI channel */
-void midi_channel_pressure(int midi, unsigned char channel,
+void midi_channel_pressure(struct midi *midi, unsigned char channel,
 			   unsigned char pressure) {
   unsigned char data[2];
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_channel_pressure() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_channel_pressure() called with null midi\n");
     return;
   }
 
   data[0]=0xd0|channel;
   data[1]=pressure&0x7f;
   
-  write(midi, data, 2);
+  write(midi->fd, data, 2);
+  fsync(midi->fd);
 }
 
 /* Sets the pitch wheel value of a MIDI channel */
-void midi_pitch_wheel(int midi, unsigned char channel,
+void midi_pitch_wheel(struct midi *midi, unsigned char channel,
 		      unsigned short value) {
   unsigned char data[3];
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_pitch_wheel() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_pitch_wheel() called with null midi\n");
     return;
   }
 
@@ -162,16 +175,17 @@ void midi_pitch_wheel(int midi, unsigned char channel,
   data[1]=(value>>7)&0x7f;
   data[2]=value&0x7f;
   
-  write(midi, data, 3);
+  write(midi->fd, data, 3);
+  fsync(midi->fd);
 }
 
 /* Sends a MIDI SysEX message */
-void midi_system_exclusive(int midi, unsigned char *message,
+void midi_system_exclusive(struct midi *midi, unsigned char *message,
 			   unsigned short length) {
   unsigned char *data=(unsigned char *)calloc(length+2, sizeof(unsigned char));
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_system_exclusive() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_system_exclusive() called with null midi\n");
     return;
   }
 
@@ -179,34 +193,36 @@ void midi_system_exclusive(int midi, unsigned char *message,
   memcpy(data+1, message, length);
   data[length+1]=0xf7;
   
-  write(midi, data, length+2);
+  write(midi->fd, data, length+2);
+  fsync(midi->fd);
 
   free(data);
 }
 
 /* Send a clock message */
-void midi_clock(int midi) {
+void midi_clock(struct midi *midi) {
   unsigned char data[]={ 0xf8 };
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_clock() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_clock() called with null midi\n");
     return;
   }
 
-  write(midi, data, 1);
+  write(midi->fd, data, 1);
+  fsync(midi->fd);
 }
 
 /* Receives a MIDI message */
-unsigned char *midi_read(int midi) {
+unsigned char *midi_read(struct midi *midi) {
   int status=1;
   unsigned char c=0, *d=NULL;
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_read() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_read() called with null midi\n");
     return NULL;
   }
 
-  while((status=read(midi, &c, 1))==1 && c<0x80);
+  while((status=read(midi->fd, &c, 1))==1 && c<0x80);
 
   if(status!=1)
     return NULL;
@@ -219,7 +235,7 @@ unsigned char *midi_read(int midi) {
   case 0xe0: {
     d=(unsigned char *)calloc(3, sizeof(unsigned char));
     d[0]=c;
-    status=read(midi, d+1, 2);
+    status=read(midi->fd, d+1, 2);
 
     if(status==2)
       return d;
@@ -235,13 +251,13 @@ unsigned char *midi_read(int midi) {
 }
 
 /* Receives a MIDI SysEx message */
-void midi_read_system_exclusive(int midi, struct sysex *sysex,
+void midi_read_system_exclusive(struct midi *midi, struct sysex *sysex,
 				int autostop) {
   int status=0, i=0;
   unsigned char c=0;
 
-  if(midi==-1) {
-    fprintf(stderr, "midi_read_system_exclusive() called with invalid fd\n");
+  if(midi==NULL) {
+    fprintf(stderr, "midi_read_system_exclusive() called with null midi\n");
     return;
   }
 
@@ -253,12 +269,12 @@ void midi_read_system_exclusive(int midi, struct sysex *sysex,
   /* Read until a SysEx dump begins */
   while(c!=0xf0) {
     usleep(40);
-    status=read(midi, &c, 1);
+    status=read(midi->fd, &c, 1);
   }
   
   /* Read until the buffer is full or to the end of the SysEx (autostop) */
   while(i<sysex->length) {
-    status=read(midi, &c, 1);
+    status=read(midi->fd, &c, 1);
     
     if(status==1) {
       sysex->data[i++]=c;
@@ -322,7 +338,7 @@ struct sysex *sysex_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
       sscanf(c, "%X", &d);
       sysex->data[i]=d;
     }
-  } else
+  } else if(cur->type!=XML_COMMENT_NODE)
     fprintf(stderr, "XML error: expected sysex, got %s\n", cur->name);
 
   return sysex;
