@@ -685,6 +685,12 @@ void song_changeinstrument(struct song *song, int from, int to, int swap) {
 void song_convert_commands(unsigned char *command,
 			   struct MMDMIDICmd3x *mmdcmd3x) {
   switch(command[0]) {
+  case COMMAND_PREVIOUS_COMMAND_VALUE:
+    if(command[1]!=0) {
+      command[0]=0x80;
+      command[1]&=0x7f;
+    }
+    break;
   case COMMAND_VELOCITY:
     /* Fix volume range to 0..127 */
     if(command[1]==64)
